#include "MatchTemplateThreadPluginNode.h"

MatchTemplateThreadPluginNode::MatchTemplateThreadPluginNode() : rvs::ThreadNode("MatchTemplateThreadPluginNode")
{
    AppendFileParameter("Img", m_img, "Load Image (*.png *.bmp *.jpg *.tiff)", "");
    AppendFileParameter("Template", m_template, "Load Image (*.png *.bmp *.jpg *.tiff)", "");
    RegisterOutput("Image", "result"); // 0
    RVS_PLUGIN_NODE_LOG(warning, "MatchTemplateThreadPluginNode generate success", "MatchTemplateThreadPluginNode 添加成功");
}

MatchTemplateThreadPluginNode::~MatchTemplateThreadPluginNode()
{
}

void MatchTemplateThreadPluginNode::ThreadInit()
{
    RVS_PLUGIN_NODE_LOG(debug, "MatchTemplateThreadPluginNode::ThreadInit()", "MatchTemplateThreadPluginNode::ThreadInit()");
}

bool MatchTemplateThreadPluginNode::BeforeProcess()
{
    RVS_PLUGIN_NODE_LOG(debug, "MatchTemplateThreadPluginNode::BeforeProcess()", "MatchTemplateThreadPluginNode::BeforeProcess()");
    m_input_image = cv::imread(m_img);
    m_input_template = cv::imread(m_template);
    return true;
}

int MatchTemplateThreadPluginNode::ThreadProcess()
{
    RVS_PLUGIN_NODE_LOG(debug, "MatchTemplateThreadPluginNode::ThreadProcess()", "MatchTemplateThreadPluginNode::ThreadProcess()");
    int result_cols = m_input_image.cols - m_input_template.cols + 1;
    int result_rows = m_input_image.rows - m_input_template.rows + 1;
    cv::Mat img_result;
    img_result.create(result_cols, result_rows, CV_32FC1);
    cv::matchTemplate(m_input_image, m_input_template, img_result, cv::TM_SQDIFF_NORMED);
    cv::normalize(img_result, img_result, 0, 1, cv::NORM_MINMAX, -1, cv::Mat());
    double minVal = -1;
    double maxVal;
    cv::Point minLoc;
    cv::Point maxLoc;
    cv::Point matchLoc;
    cv::minMaxLoc(img_result, &minVal, &maxVal, &minLoc, &maxLoc, cv::Mat());
    matchLoc = minLoc;
    cv::Point center = cv::Point(minLoc.x + m_input_template.cols / 2, minLoc.y + m_input_template.rows / 2);
    cv::rectangle(m_input_image, matchLoc, cv::Point(matchLoc.x + m_input_template.cols, matchLoc.y + m_input_template.rows), cv::Scalar(0, 255, 0), 2, 8, 0);
    cv::circle(m_input_image, center, 2, cv::Scalar(255, 0, 0), 2);
    return 1;
}

void MatchTemplateThreadPluginNode::ThreadFinish()
{
}

bool MatchTemplateThreadPluginNode::AfterProcess()
{
    RVS_PLUGIN_NODE_LOG(debug, "MatchTemplateThreadPluginNode::AfterProcess()", "MatchTemplateThreadPluginNode::AfterProcess()");
    Image *m_result = GetImageOutput(0);
    *m_result = m_input_image;
    return true;
}