#include "SimpleSortPluginNode.h"

SimpleSortPluginNode::SimpleSortPluginNode()
    : rvs::Node("SimpleSortPluginNode")
// , m_sort_mode_name_list { "xy", "yx"}
// , m_sort_mode_name(m_sort_mode_name_list[0])
// , m_sort_mode_list{"Descending","Ascending"}
// , m_sort_mode(m_sort_mode_list[0])
{
  // append parameter
  m_sort_mode_name_list = {"xy", "yx"};
  m_sort_mode_list = {"Descending", "Ascending"};
  AppendEnumParameter(m_sort_mode_name_list, "sort_mode", m_sort_mode_name,
                      m_sort_mode_name_list[0]);
  AppendEnumParameter(m_sort_mode_list, "sort_x_mode", m_sort_x_mode, m_sort_mode_list[0]);
  AppendEnumParameter(m_sort_mode_list, "sort_y_mode", m_sort_y_mode, m_sort_mode_list[0]);
  AppendFloatParameter("x_dif_range", x_dif_range, 0.1f);
  AppendFloatParameter("y_dif_range", y_dif_range, 0.1f);

  // register input
  RegisterInput("Pose", "input_poselist", true); // 0

  // register output
  RegisterOutput("Pose", "output_poselist", true); // 0
}

SimpleSortPluginNode::~SimpleSortPluginNode() {}

int SimpleSortPluginNode::Process()
{
  // get input_poselist
  const PoseList *input_poselist = GetPoseInputList(0);
  if (input_poselist == NULL || input_poselist->empty())
  {
    RVS_PLUGIN_NODE_LOG(warning, "input_poselist is null or empty", "input_poselist输入为空");
    return 0;
  }

  //
  std::vector<Pose> poselist = *input_poselist;
  std::sort(poselist.begin(), poselist.end(), [this](const Pose &p1, const Pose &p2)
            { return Compare(p1, p2); });
  // get out_poselist
  std::vector<Pose> *output_poselist = GetPoseOutputList(0);
  output_poselist->clear();
  for (unsigned int i = 0; i < poselist.size(); i++)
  {
    output_poselist->push_back(poselist[i]);
  }
  return 1;
}

bool SimpleSortPluginNode::Compare(const Pose &pose1, const Pose &pose2)
{
  if (m_sort_mode_name == "xy")
  {
    if (abs(pose1.x - pose2.x) <= x_dif_range)
    {
      if (m_sort_y_mode == "Ascending")
        return pose1.y < pose2.y;
      else
        return pose1.y > pose2.y;
    }
    else
    {
      if (m_sort_x_mode == "Ascending")
        return pose1.x < pose2.x;
      else
        return pose1.x > pose2.x;
    }
  }
  else if (m_sort_mode_name == "yx")
  {
    if (abs(pose1.y - pose2.y) <= y_dif_range)
    {
      if (m_sort_x_mode == "Ascending")
        return pose1.x < pose2.x;
      else
        return pose1.x > pose2.x;
    }
    else
    {
      if (m_sort_y_mode == "Ascending")
        return pose1.y < pose2.y;
      else
        return pose1.y > pose2.y;
    }
  }
  else
  {
    RVS_PLUGIN_NODE_LOG(warning, "m_sort_field_name: not supported", "sort_mode:不支持");
    return false;
  }
  return false;
}
